From 34727abb66a01d2f21df0bc612d3a169b64c9a16 Mon Sep 17 00:00:00 2001
From: MT <mtekman89@gmail.com>
Date: Tue, 2 May 2023 17:24:56 +0200
Subject: [PATCH 1/6] lisp/ob-tangle.el: Detangle a single block

* ob-tangle.el (org-babel-detangle--block-contents,
org-babel-detangle-single-block,
org-babel-detangle-single-block--from-source): Detangling the contents
of a single target tangled block back to the original source org mode
block, in place.

(org-babel-detangle--block-contents): Within the comments block of a
remote tangled file, return its body.  If a NEW-BODY argument is
given, then replace the body.

(org-babel-detangle-single-block): Interactive function to detangle
the body of the remote tangled file back into its org mode source
block.  It is invoked from inside the comments block of the tangled
file, and relies on `org-babel-detangle--block-contents' for copying
contents back to source. The value returned is the action that was
performed on the block, which is one of "import" (remote to source),
"export" (source to remote) or "both" (symmetric).

(org-babel-detangle-single-block--from-source): Produces the same
result as `org-babel-detangle-single-block', but is instead invoked
from inside the org mode source block.  It relies on
`org-babel-detangle-single-block' to do the actual detangling.
---
 lisp/ob-tangle.el | 72 +++++++++++++++++++++++++++++++++++++++++++++++
 1 file changed, 72 insertions(+)

diff --git a/lisp/ob-tangle.el b/lisp/ob-tangle.el
index 980d4a120..c6a4d8448 100644
--- a/lisp/ob-tangle.el
+++ b/lisp/ob-tangle.el
@@ -621,6 +621,52 @@ which enable the original code blocks to be found."
   (interactive)
   (save-excursion
     (when source-code-file (find-file source-code-file))
+
+(defun org-babel-detangle-single-block (&optional action ignore-header)
+  "Propagate changes in tangled file with comments back to the
+original source code block in the Org file, whilst respecting the
+`:tangle-sync' action header, unless ACTION is specified, or
+IGNORE-HEADER is t.  Return the tangle-sync action performed."
+  (interactive)
+  (save-window-excursion
+    (let* ((tang-buff (current-buffer))
+           (new-body (org-babel-tangle-jump-to-org))
+           (block-info (org-babel-get-src-block-info 'no-eval))
+           (sync-action (or action (cdr (assq :tangle-sync (nth 2 block-info))))))
+      (cond ((or ignore-header
+                 (string= sync-action "import")
+                 (string= sync-action "both")
+                 (null sync-action))
+             (org-babel-update-block-body new-body))
+            ((string= sync-action "skip") nil)
+            ((string= sync-action "export")
+             ;; Perform in-place tangling without writing to a new
+             ;; buffer. Clean, but bypasses tangle hooks and other
+             ;; tangle-related functions.
+             (with-current-buffer tang-buff
+               (org-babel-detangle--block-contents nil (nth 1 block-info)))))
+      sync-action)))
+
+(defun org-babel-detangle-single-block--from-source ()
+  "Detangle from Org mode source buffer.
+Propagate the remote changes of a tangled file to the current
+source code block under cursor.  Assumes that the block is
+requesting a data import and ignores the tangle-sync header."
+  (pcase-let*
+      ((spec (cdadar (org-babel-tangle-single-block 1 t)))
+       ;; Adapted from `org-babel-spec-to-string'
+       (`(,start ,file ,link ,source ,info ,_body ,_comment) spec)
+       (tangle-file (cdr (assq :tangle info)))
+       (link-data `(("start-line" . ,(number-to-string start))
+		    ("file" . ,file)
+		    ("link" . ,link)
+		    ("source-name" . ,source)))
+       (link-text (org-fill-template org-babel-tangle-comment-format-beg link-data)))
+    (with-current-buffer (find-file-noselect tangle-file)
+      (goto-char (point-min))
+      (search-forward link-text nil t)
+      (org-babel-detangle-single-block))))
+
     (goto-char (point-min))
     (let ((counter 0) new-body end)
       (while (re-search-forward org-link-bracket-re nil t)
@@ -637,6 +683,32 @@ which enable the original code blocks to be found."
         (goto-char end))
       (prog1 counter (message "Detangled %d code blocks" counter)))))
 
+(defun org-babel-detangle--block-contents (&optional nearest new-body)
+  "Get the contents of the current exported block under cursor, or NEAREST if t.
+If NEW-BODY is given, then update the block contents in place."
+  ;; Adapted body from org-babel-detangle
+  (save-excursion
+    (move-end-of-line 1) ;; for backwards regex to work
+    (let* ((cursor (+ 2 (line-beginning-position)))
+           (block-end nil)
+           (body-end nil)
+           (block-start (progn (re-search-backward org-link-bracket-re nil t)
+                               (line-beginning-position)))
+	   (body-start (line-beginning-position 2))
+	   (block-name (match-string 2)))
+      (when block-start
+	(re-search-forward
+	 (concat " " (regexp-quote block-name)
+		 " ends here")
+	 nil t)
+	(setq body-end (line-beginning-position)
+              block-end (line-end-position)))
+      (when (or nearest (and (>= block-end cursor) (>= cursor block-start)))
+        (if new-body
+            (cl--set-buffer-substring body-start body-end (concat new-body "\n"))
+          (buffer-substring body-start body-end))))))
+
+
 (defun org-babel-tangle-jump-to-org ()
   "Jump from a tangled code file to the related Org mode file."
   (interactive)
-- 
2.40.1

